﻿# -*- coding: utf-8 -*-
"""
bp.pyプログラム
バックプロパゲーションによる階層型ニューラルネットの学習
誤差の推移や，学習結果となる結合係数などを出力します
使い方　c:\>python bp.py 
"""
# モジュールのインポート
import math
import sys
import random

# グローバル変数
INPUTNO = 5       #入力層のニューロン数
HIDDENNO = 3      #中間層のニューロン数
OUTPUTNO = 5      #出力層のニューロン数
ALPHA = 10        #学習係数
SEED = 65535      #乱数のシード
MAXINPUTNO = 100  #データの最大個数
BIGNUM = 100.0    #誤差の初期値
LIMIT = 0.001     #誤差の上限値

# 下請け関数の定義
# forward()関数
def forward(wh,wo,hi,e):
    """順方向の計算"""
    #hiの計算
    for i in range(HIDDENNO):
        u = 0.0
        for j in range(INPUTNO):
            u += e[j] * wh[i][j]
        u -= wh[i][INPUTNO] #しきい値の処理
        hi[i] = f(u)
    #出力oの計算
    o=0.0
    for i in range(HIDDENNO):
        o += hi[i] * wo[i]
    o -= wo[HIDDENNO] #しきい値の処理
    return f(o)
# forward()関数の終わり

# f()関数
def f(u):
    """伝達関数"""
    #シグモイド関数の計算
    return 1.0/(1.0 + math.exp(-u))
 
# f()関数の終わり

# olearn()関数
def olearn(wo,hi,e,o,k):
    """出力層の重み学習"""
    #誤差の計算
    d = (e[INPUTNO + k] - o) * o * (1 - o)
    #重みの学習
    for i in range(HIDDENNO):
        wo[i] += ALPHA * hi[i] * d
    #しきい値の学習
    wo[HIDDENNO] += ALPHA * (-1.0) * d
    return 
# olearn()関数の終わり

# hlearn()関数
def hlearn(wh,wo,hi,e,o,k):
    """中間層の重み学習"""
    #中間層の各セルjを対象
    for j in range(HIDDENNO):
        dj = hi[j] * (1 - hi[j]) * wo[j] * (e[INPUTNO + k] - o) * o * (1 - o)
        #i番目の重みを処理
        for i in range(INPUTNO):
            wh[j][i] += ALPHA * e[i] * dj
        #しきい値の学習
        wh[j][INPUTNO] += ALPHA * (-1.0) * dj
    return 
# hlearn()関数の終わり

# メイン実行部
#乱数の初期化
random.seed(SEED)

#変数の準備
wh = [[random.uniform(-1,1) for i in range(INPUTNO + 1)]
       for j in range(HIDDENNO)]        #中間層の重み
wo = [[random.uniform(-1,1) for i in range(HIDDENNO + 1)]
       for j in range(OUTPUTNO)]        #出力層の重み
hi = [0.0 for i in range(HIDDENNO + 1)] #中間層の出力
o = [0.0 for i in range(OUTPUTNO)]      #出力
err = BIGNUM                            #誤差の評価
#学習データセット
e = [[1, 0, 0, 0, 0, 1, 0, 0, 0, 0] ,
     [0, 1, 0, 0, 0, 0, 1, 0, 0, 0] ,
     [0, 0, 1, 0, 0, 0, 0, 1, 0, 0] ,
     [0, 0, 0, 1, 0, 0, 0, 0, 1, 0] ,
     [0, 0, 0, 0, 1, 0, 0, 0, 0, 1]]
n_of_e = len(e)

#結合荷重の初期値の出力
print(wh, wo)

#学習
count = 0
while err > LIMIT :
    #複数の出力層に対応
    for k in range(OUTPUTNO):
        err = 0.0
        for j in range(n_of_e):
            #順方向の計算
            o[k] = forward(wh, wo[k], hi, e[j])
            #出力層の重みの調整
            olearn(wo[k], hi, e[j], o[k], k)
            #中間層の重みの調整
            hlearn(wh, wo[k], hi, e[j], o[k], k)
            #誤差の積算
            teacherno = INPUTNO + k
            err += (o[k] - e[j][teacherno]) * (o[k] - e[j][teacherno]) 
        count += 1
        #誤差の出力
        print(count," ",err)
#結合荷重の出力
print(wh,wo)

#学習データに対する出力 
for i in range(n_of_e):
    print(i)
    print(e[i])
    outputlist = []
    for j in range(OUTPUTNO):
        outputlist.append(forward(wh, wo[j], hi, e[i]))
    print(['{:.3f}'.format(num) for num in outputlist])
# bp.pyの終わり
