﻿# -*- coding: utf-8 -*-
"""
pso.pyプログラム
粒子群最適化法による関数の極値探索プログラム
結果をグラフ描画します
使い方　c:\>python pso.py
"""
# モジュールのインポート
import random
import numpy as np
import matplotlib.pyplot as plt

# 定数
N = 30           # 粒子の個数
TIMELIMIT = 50  # シミュレーション打ち切り時刻
W = 0.3          # 慣性定数
C1 = 1.2         # ローカルな質量
C2 = 1.2         # グローバルな質量
SEED = 65535     # 乱数の種

# クラス定義
# Particleクラス
class Particle:
    """粒子を表現するクラスの定義"""
    def __init__(self):  # コンストラクタ
        self.x = random.uniform(-2.0, 2.0)  # x座標の初期値
        self.y = random.uniform(-2.0, 2.0)  # y座標の初期値
        self.value = calcval(self.x, self.y)# 評価値
        self.bestval = self.value           # 最適値
        self.vx = random.uniform(-1.0, 1.0) # 速度のx成分の初期値
        self.vy = random.uniform(-1.0, 1.0) # 速度のy成分の初期値
        self.bestpos_x = self.x             # 最適位置（x座標)
        self.bestpos_y = self.y             # 最適位置（y座標)

    def optimize(self):  # 次時刻の状態の計算
        r1 = random.random()   # 乱数r1の設定
        r2 = random.random()   # 乱数r2の設定
        # 速度の更新
        self.vx = W * self.vx \
                + C1 * r1 * (self.bestpos_x - self.x) \
                + C2 * r2 * (gbestpos_x - self.x)
        self.vy = W * self.vy \
                + C1 * r1 * (self.bestpos_y - self.y) \
                + C2 * r2 * (gbestpos_y - self.y)
        # 位置の更新
        self.x += self.vx
        self.y += self.vy
        # 最適値の更新
        self.value = calcval(self.x, self.y)
        if self.value < self.bestval:
            self.bestval = self.value
            self.bestpos_x = self.x
            self.bestpos_y = self.y

# Particleクラスの定義の終わり

# 下請け関数の定義
# calcval()関数
def calcval(x, y):
    """評価値の計算"""
    return x * x + y * y

# calcval()関数の終わり

# setgbest()関数
def setgbest():
    """群中の最適位置と最適値の設定"""
    global gbestval
    global gbestpos_x
    global gbestpos_y
    for i in range(N):
        if ps[i].value < gbestval :
            gbestval = ps[i].bestval
            gbestpos_x = ps[i].bestpos_x
            gbestpos_y = ps[i].bestpos_y
# setgbest()関数の終わり

# メイン実行部
# 初期化
random.seed(SEED)  # 乱数の初期化
# 粒子群の生成
ps = [Particle() for i in range(N)]
# 群中の最適位置と最適値の設定
gbestpos_x = ps[0].bestpos_x
gbestpos_y = ps[0].bestpos_y
gbestval = ps[0].bestval
setgbest()      

# グラフデータの初期化
xlist = []
ylist = []

# 探索
for t in range(TIMELIMIT):
    print("t = ",t)
    for i in range(N):
        ps[i].optimize()  # 次時刻の状態を計算
        setgbest()
        # グラフデータの追加     
        xlist.append(ps[i].x)
        ylist.append(ps[i].y)
    # グラフの表示
    plt.clf()  # グラフ領域のクリア
    plt.axis([-2, 2, -2, 2])   # 描画領域の設定
    plt.plot(xlist, ylist, ".")  # プロット
    plt.pause(0.01)
    # 描画データのクリア
    xlist.clear()
    ylist.clear() 
plt.show()       
# pso.pyの終わり
