﻿# -*- coding: utf-8 -*-
"""
filter.pyプログラム
空間フィルタの適用  
2次元データを読み取り、空間フィルタを適用します
使い方　c:\>python filter.py < data1.txt
"""
# モジュールのインポート
import math
import sys

# グローバル変数
INPUTSIZE = 11    #入力数

# 下請け関数の定義
# getdata()関数
def getdata(im):
    """画像データの読み込み"""
    n_of_e = 0   #データセットの行数
    # データの入力
    for line in sys.stdin : 
        im[n_of_e] = [float(num) for num in line.split()]
        n_of_e += 1
    return 
# getdata()関数の終わり

# filtering()関数
def filtering(filter,im,im_out):
    """フィルタの適用"""
    for i in range(1,INPUTSIZE - 1):
            for j in range(1,INPUTSIZE - 1):
               im_out[i][j] = calcfilter(filter,im,i,j)
    return
# filtering()関数の終わり

# calcfilter()関数
def calcfilter(filter,im,i,j):
    """フィルタの適用"""
    sum = 0.0
    for m in range(3):
            for n in range(3):
                sum += im[i - 1 + m][j - 1 + n] * filter[m][n]
    if sum < 0 : #結果が負の場合は0とする
        sum = 0
    return sum
# calcfilter()関数の終わり

# メイン実行部
np = 1.0 / 9.0

filter = [[0,1,0],[1,-4,1],[0,1,0]]           #ラプラシアンフィルタ
#filter = [[np,np,np],[np,np,np],[np,np,np]]  #平均フィルタ

im = [[0.0 for i in range(INPUTSIZE)] 
      for j in range(INPUTSIZE)]             #入力データ
im_out = [[0.0 for i in range(INPUTSIZE)] 
            for j in range(INPUTSIZE)]       #出力画像

#入力データの読み込み
getdata(im)

#フィルタの適用
filtering(filter,im,im_out)

#結果の出力
for i in im_out:
    for j in i:
        print("{:.3f} ".format(j),end = "")
    print()

# filter.pyの終わり
